@@ -3,8 +3,8 @@
 #include <QDebug>
 #include <math.h>
 #include "glwidget.h"
-#include <vector>
 #include <GLES2/gl2.h>
+#include <vector>
 #include <stdlib.h>
 #include <mce/dbus-names.h>
 #include <mce/mode-names.h>
@@ -85,6 +85,7 @@ void GLWidget::keyReleaseEvent(QKeyEvent *event) {
 
 
 GLWidget::GLWidget(QMainWindow *parent) : QGLWidget(parent) {
+	tempArray = new GLfloat[16];
 	first_draw = true;
 	setAutoFillBackground(false);
 	setAutoBufferSwap(false);
@@ -134,12 +135,12 @@ void GLWidget::newGame() {
 	xPos = zPos = suuntaX = suuntaY = xRot = yRot = zRot = 0.0f;
 	//xPos = 60.0;
 	//zPos = 60.0;
-	suuntaX = 60.0f;
+	suuntaX = 40.0f;
 	s1 = s2 = 0.0f;
 	lopussa = false;	
 	//sienien arvonnat ja lisays
 	sienipaikat.clear();
-	for(int i = 0; i<50;i++) {
+	for(int i = 0; i<22;i++) {
 		sPaikka tmp;
 		tmp.x = (float)(qrand() % 380+10.0f);
 		tmp.z = (float)(qrand() % 380+10.0f);
@@ -171,21 +172,17 @@ void GLWidget::paivita() {
 	s1 += 0.1f;
 	s2 -= 0.3f;
 		
-	suuntaX -= xax*0.015;
-	kaasupohjassa ? speed = 1.0f : speed *= 0.5f;
+	suuntaX -= xax*0.01;
+	kaasupohjassa ? speed = 1.3f : speed *= 0.7f;
 	try {
 		 aY = qRed(qim.pixel(aX, aZ))/255.0f*100+5.0f;
 	}	
 	catch (exception ex) {
 		aY = yPos = 6.0f;
 	}
-	aX += cos(suuntaX*M_PI/180)*speed;
-	aZ -= sin(suuntaX*M_PI/180)*speed;
+	aX -= cos(-suuntaX*M_PI/180)*speed;
+	aZ -= sin(-suuntaX*M_PI/180)*speed;
 
-	yPos = aY + 40.0f;
-	xPos = aX;
-	zPos = aZ - 20.0f;
-	
 	update();
 
 
@@ -209,7 +206,7 @@ void GLWidget::initializeGL()
 	qim = QImage(fname);
 	QString nfname = ":/textures/dpngn.png";
 	QImage qnmap = QImage(nfname);
-	textures = new GLuint[1];		
+	textures = new GLuint[2];		
 	tbuffers = new GLuint[2];
 	glGenBuffers(2, &tbuffers[0]);
 	string nammi = "/opt/mushrooms/md2model.md2";
@@ -270,15 +267,27 @@ void GLWidget::initializeGL()
 	free(indices);	
 		
 	QImage bufi;
-	bufi.load(QString(":/textures/terrain.png"));
+	bufi.load(QString(":/textures/grass.jpg"));
 	textImage = QGLWidget::convertToGLFormat(bufi);	
-	glGenTextures(1, textures);
+	glGenTextures(2, textures);
 	glBindTexture(GL_TEXTURE_2D, textures[0]);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, textImage.height(), textImage.width(), 0, GL_RGBA, GL_UNSIGNED_BYTE, textImage.bits());
+	glBindTexture(GL_TEXTURE_2D, 0);	
+	
+	/*toinen textuuri*/
+	bufi.load(QString(";/textures/sieni.png"));
+	textImage = QGLWidget::convertToGLFormat(bufi);
+	glBindTexture(GL_TEXTURE_2D, textures[1]);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, textImage.height(), textImage.width(), 0, GL_RGBA, GL_UNSIGNED_BYTE, textImage.bits());
 	glBindTexture(GL_TEXTURE_2D, 0);	
 	
+
+
+	
 	string name = ":/models/sieni.obj";
 	string textname = ":/textures/sieni.png";
 	sieni = new Model(name.c_str(), textname.c_str());
@@ -289,31 +298,22 @@ void GLWidget::initializeGL()
 	
 	GLuint vshader = glCreateShader(GL_VERTEX_SHADER);
 		const char *vsrc[1] = {
-			"attribute highp vec4 vertex;\n"
+			"attribute mediump vec4 vertex;\n"
 			"attribute mediump vec2 texture; \n"
 			"attribute mediump vec3 normaali; \n"
-			"uniform mat4 rotmatrix;\n"
 			
 			"varying mediump vec2 v_tex; \n"
 			"uniform mediump mat4 matrix;\n"
 			"uniform mediump mat4 projection; \n"
-			"uniform highp vec4 transvec; \n"
-			"uniform highp float scale; \n"
+			"uniform mediump float scale; \n"
 			
        		"void main()                 \n"
        		"{                           \n"
        		"   	mat4 paikka = projection*matrix; \n"
-		"	vec4 apu = vertex*rotmatrix;		\n"
-		"	apu.x *= scale;			\n"
-		"	apu.y *= scale;			\n"
-		"	apu.z *= scale;			\n"
-		"	vec4 tapu = transvec;		\n"
-		"	tapu.x *= 2.0;		\n"
-		"	tapu.y *= 2.0;		\n"
-		"	tapu.z *= 2.0;		\n"
-		
+		"	vec4 apu = vertex;		\n"
+		"	apu.xyz *= scale;			\n"
 		"					\n"
-		"	gl_Position =paikka*(apu+tapu);		\n"
+		"	gl_Position =paikka*apu;		\n"
 		"			\n"
 		"	\n"
 		" 	v_tex = texture;				\n"
@@ -329,14 +329,6 @@ void GLWidget::initializeGL()
 		"void main()                                \n"
 		"{                                          \n"
 		" mediump vec4 color = texture2D(sampler, v_tex); \n"
-               /* " mediump vec3 N = normalize(v_N); \n"
-                " mediump vec3 V = normalize(v_V); \n"
-                " mediump vec3 R = reflect(V, N);  \n"
-                " mediump vec3 L = normalize(lightDir);                 \n"
-                " mediump vec4 ambient = vec4(0.1, 0.1, 0.1, 1.0);      \n"
-                " mediump vec4 diffuse = vec4(0.4,0.4,0.4,1.0) *  max(dot(L, N), 0.0);  \n"
-                " mediump vec4 specular = vec4(0.7, 0.7, 0.7, 1.0);     \n"
-                " specular = specular*pow(max(dot(R, L), 0.0), 1.4);    \n" */
                 " gl_FragColor = color;     \n"
 		"}\n"
 		};
@@ -396,12 +388,9 @@ void GLWidget::initializeGL()
 		
 		
 		
-		matrixUniform1 = glGetUniformLocation(program, "matrix");
-		matrixUniform2 = glGetUniformLocation(program, "projection");
-		transUniform1 = glGetUniformLocation(program, "transvec");
-		scaleUniform1 = glGetUniformLocation(program, "scale");	
-		nmatrixUniform = glGetUniformLocation(program, "normalmatrix");
-		rotmUniform = glGetUniformLocation(program, "rotmatrix");
+		matrix = glGetUniformLocation(program, "matrix");
+		projectionU = glGetUniformLocation(program, "projection");
+		scale = glGetUniformLocation(program, "scale");	
 		
 		QTime ftime(0,0,0);
 		qsrand(ftime.secsTo(QTime::currentTime()));
@@ -427,49 +416,51 @@ void GLWidget::resizeGL(int w, int h) {
 	glViewport(0,0, (GLint)w, (GLint)h); 
 	projection = new GLfloat[16];
 	BuildPerspProjMat(projection, 3.141/4.0f, this->width()/this->height(), -1.0f, 1000.0f);
-		
-}
-
+	}
 
 void GLWidget::paintEvent(QPaintEvent *event) {
-	if(first_draw) {
-		QGLWidget::paintEvent(event);
-		first_draw = false;
-	}
+	QPainter painter;
+	painter.begin(this);
 	makeCurrent();
-	QPainter painter(this);
 	glClearColor(0.2f, 0.2f, 0.8f, 1.0f);
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 	glEnable(GL_DEPTH_TEST);
 	GLfloat modelview[16];
-	GLfloat rotMatrix[16];	
-       	LookAt(modelview, -xPos, -yPos, -zPos, -aX, -yPos+20.0f, -aZ, 0, 1, 0);
+	LookAt(modelview, 0, 0, 0, 60, 50, 0, 0, 1, 0);
 	glUseProgram(program);
-	glUniformMatrix4fv(matrixUniform2, 1, GL_FALSE, projection);
-	
-	TranslateMatrix(modelview, -xPos,-yPos,-zPos);	
-
-	LoadIdentityMatrix(rotMatrix);	
-	glUniformMatrix4fv(rotmUniform, 1, GL_FALSE, rotMatrix);	
-	glUniformMatrix4fv(matrixUniform1, 1, GL_FALSE, modelview);
-	
-	glUniform1f(scaleUniform1, 1.0);
-	glUniform4f(transUniform1, 0.0, .0, 0.0, 0.0); 
+//projection
+	glUniformMatrix4fv(projectionU, 1, GL_FALSE, projection);
+	RotateMatrix(modelview, -12, 0, 0, 1);
+	TranslateMatrix(modelview, -40,-30,0);	
+	RotateMatrix(modelview, -suuntaX, 0,1,0);
+	TranslateMatrix(modelview, -aX, -aY-10, -aZ);
+		
+	glUniformMatrix4fv(matrix, 1, GL_FALSE, modelview);
+	glUniform1f(scale, 1.0);
 
 	Plane();
-	
+		
+  {GLenum error = glGetError();
+	   if(error != GL_NO_ERROR) {
+	        qDebug() << error << "toka";
+	  }}
+		
 	for(list<sPaikka>::iterator pos = sienipaikat.begin(); pos!=sienipaikat.end(); ++pos ) {
-		glUniform1f(scaleUniform1, pos->s);
-		glUniform4f(transUniform1, pos->x, pos->y,pos->z, 1.0);
+		memcpy(tempArray, modelview, sizeof(GLfloat)*16);
+		TranslateMatrix(modelview, pos->x, pos->y,pos->z);
+		RotateMatrix(modelview, -0.0f, 1, 0, 0);
+		glUniform1f(scale, pos->s);
+		glUniformMatrix4fv(matrix, 1, GL_FALSE, modelview);	
 		sieni->render(program);
+		memcpy(modelview, tempArray, sizeof(GLfloat)*16);
 	}
-	LoadIdentityMatrix(rotMatrix);	
-	glUniform1f(scaleUniform1, 1.0);
-	RotateMatrix(rotMatrix, 90.0f, 1.0, 0.0, 0.0);
-	RotateMatrix(rotMatrix, -suuntaX+90.0f, 0.0, 1.0, 0.0);
-	glUniform4f(transUniform1, aX,aY,aZ, 1.0);
-	glUniformMatrix4fv(rotmUniform, 1, GL_FALSE, rotMatrix);	
-	glUniformMatrix4fv(matrixUniform1, 1, GL_FALSE, modelview);
+	
+	
+	TranslateMatrix(modelview, aX,aY,aZ);
+	RotateMatrix(modelview, 90+suuntaX, 0.0, 1.0, 0.0);
+	RotateMatrix(modelview, -90, 1, 0, 0);
+	glUniform1f(scale, 0.6);
+	glUniformMatrix4fv(matrix, 1, GL_FALSE, modelview);
 	md2->RenderBuffers(keyIndex, program);
 			
 	
@@ -501,14 +492,14 @@ void GLWidget::paintEvent(QPaintEvent *event) {
 	QString aikas;
 	
 	framesPerSecond.setNum(frames /(time.elapsed() / 1000.0), 'f', 2);
-	keratyt.setNum(50-sienipaikat.size());
+	keratyt.setNum(22-sienipaikat.size());
 	aikas.setNum(ajankesto);
 	painter.drawText(QPointF(550, 30), keratyt + "/20");
 	painter.drawText(QPointF(400, 30), aikas+ "s"); 
 	painter.drawText(QPointF(20, 30), framesPerSecond + " fps");
 	
 	
-	if(sienipaikat.size() <= 30) {
+	if(sienipaikat.size() <= 2) {
 		if(!lopussa) {
 			pauseGame();
 			lopussa = true;
@@ -522,8 +513,6 @@ void GLWidget::paintEvent(QPaintEvent *event) {
 	}
 	painter.end();
 	swapBuffers();
-
-
 }
 	
 
@@ -546,6 +535,7 @@ void GLWidget::Plane() {
 	glDrawElements(GL_TRIANGLE_STRIP, pmitta, GL_UNSIGNED_INT, (GLvoid *)((char *)NULL));
 	glBindTexture(GL_TEXTURE_2D, 0);
 	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
 	glDisableVertexAttribArray(0);
 	glDisableVertexAttribArray(1);
 	glDisableVertexAttribArray(2);
