@@ -1,7 +1,8 @@
 #ifndef glwidget_h
 #define glwidget_h
+
 #include <QGLWidget>
-//#include <QtOpenGL>
+#include <QtOpenGL>
 #include <QDBusInterface>
 #include <vector>
 #include "model.h"
@@ -71,14 +72,9 @@ private:
 	std::list<sPaikka> sienipaikat;
 	Model *sieni;
 	GLuint *textures, *tbuffers;
-    	GLuint vertexAttr1;
-    	GLuint vertexAttr2;
-    	GLuint textureAttr;
-    	GLuint matrixUniform1, gmatrixu, timeUniform, gmatrixu2, nmatrixUniform, rotmUniform;
-    	GLuint matrixUniform2;
-    	GLuint textureUniform;
-	GLuint transUniform1;
-	GLuint scaleUniform1;
+    	GLuint projectionU, matrix, timeU;
+	GLfloat *tempArray;
+	GLuint scale;
 	int mx;
 	int my;
 	void paintModel();
